//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  LimeGreen
#property indicator_color2  DimGray
#property indicator_color3  DimGray
#property indicator_color4  DimGray
#property indicator_style2  STYLE_DASH
#property indicator_style3  STYLE_DASH
#property indicator_style4  STYLE_DASH
#property indicator_width1  2
#property indicator_level1   40
#property indicator_level2  -40
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor Gold

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    ShortLimit       = 10;
extern int    LongLimit        = 20;
extern int    CfbNormLength    = 50;
extern int    CfbDepth         = 6;
extern int    CfbPrice         = PRICE_WEIGHTED;
extern int    CfbSmooth        = 8;
extern int    SmoothLength     = 5;
extern double SmoothPhase      = 0;
extern bool   SmoothDouble     = true;
extern int    mamode           = MODE_LWMA;
extern int    BandsPeriod      = 20;
extern double BandsDeviations  = 3.25;
extern bool   Delayed          = false;

extern bool   ShowArrows       = false;
extern string ArrowsIdentifier = "wprsmoothed";
extern color  ArrowUpColor     = Aqua;
extern int    ArrowUpWidth     = 2;
extern color  ArrowDownColor   = Red; 
extern int    ArrowDownWidth   = 2; 
extern bool   Interpolate      = true;

extern string  _               = "alerts settings";
extern bool    alertsOn        = false;
extern bool    alertsOnCurrent = true;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = false;
extern bool    alertsEmail     = false;

//
//
//
//
//

double wpr[];
double bandsUp[];
double bandsMi[];
double bandsDo[];
double trend[];
double atrend[];
double cfb[];

string IndicatorFileName;
bool   Calculatingwpr;
bool   ReturningBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,wpr);
   SetIndexBuffer(1,bandsUp);
   SetIndexBuffer(2,bandsMi);
   SetIndexBuffer(3,bandsDo);
   SetIndexBuffer(4,cfb);
   SetIndexBuffer(5,trend);
   SetIndexBuffer(6,atrend);
   
      //
      //
      //
      //
      //
   
      IndicatorFileName   = WindowExpertName();
      Calculatingwpr      = (TimeFrame=="Calculatewpr"); if (Calculatingwpr) return(0);
      ReturningBars       = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame           = stringToTimeFrame(TimeFrame);
         
      //
      //
      //
      //
      //
               
   IndicatorShortName("cfb adaptive wpr with bands "+timeFrameToString(timeFrame)+" ("+ShortLimit+","+LongLimit+","+BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}

//
//
//
//
//

int deinit()
{
   if (ShowArrows)
   {
      int compareLength = StringLen(ArrowsIdentifier);
      for (int i=ObjectsTotal(); i>= 0; i--)
      {
         string name = ObjectName(i);
            if (StringSubstr(name,0,compareLength) == ArrowsIdentifier) ObjectDelete(name);  
      }
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars) { wpr[0] = limit+1; return(0); }

   //
   //
   //
   //
   //   
   
   if (Calculatingwpr || timeFrame==Period())
   {
   
      //
      //
      //
      //
      //
      
      for(i=limit, r=Bars-i-1; i>=0; i--, r++)
      {
         if (Delayed)
         {
              cfb[i+1] = iDSmooth(iCfb(iMA(NULL,0,1,0,MODE_SMA,CfbPrice,i+1),CfbDepth,CfbSmooth,i+1),SmoothLength,SmoothPhase,i+1);
              cfb[i]   = iDSmooth(iCfb(iMA(NULL,0,1,0,MODE_SMA,PRICE_OPEN,i),CfbDepth,CfbSmooth,i  ),SmoothLength,SmoothPhase,i);
         }
         else cfb[i]   = iDSmooth(iCfb(iMA(NULL,0,1,0,MODE_SMA,CfbPrice,i),CfbDepth,CfbSmooth,i),SmoothLength,SmoothPhase,i);
         
         //
         //
         //
         //
         //
         
         double cfbMax = cfb[ArrayMaximum(cfb,CfbNormLength,i)];
         double cfbMin = cfb[ArrayMinimum(cfb,CfbNormLength,i)];
         double denominator = cfbMax-cfbMin;
            if (denominator> 0)
                  double ratio = (cfb[i]-cfbMin)/denominator;
            else         ratio = 0.5;                 
               
            //
            //
            //
            //
            //

            double WprPeriod = MathCeil(ShortLimit+ratio*(LongLimit-ShortLimit));
               if (Delayed)
               {
                  double dClose   = Close[i+1];
                  double dMaxHigh = High[iHighest(NULL,0,MODE_HIGH,WprPeriod, i+1)];
                  double dMinLow  = Low[iLowest(NULL, 0, MODE_LOW, WprPeriod, i+1)];
                     if (dMaxHigh != dMinLow)
                        double dummy = iDSmooth(50+(-100)*(dMaxHigh - dClose) / (dMaxHigh - dMinLow),SmoothLength,SmoothPhase,i+1,20);
                  
                         dClose   = Open[i];
                         dMaxHigh = High[iHighest(NULL,0,MODE_HIGH,WprPeriod-1,i+1)];
                         dMinLow  = Low[iLowest(NULL, 0, MODE_LOW, WprPeriod-1,i+1)];  
                         dMaxHigh = MathMax(dMaxHigh,Open[i]);
                         dMinLow  = MathMin(dMinLow ,Open[i]);
               }
               else
               {
                  dClose    = Close[i];
                  dMaxHigh  = High[iHighest(NULL,0,MODE_HIGH,WprPeriod, i)];
                  dMinLow   = Low[iLowest(NULL, 0, MODE_LOW, WprPeriod, i)];  
               }            
               wpr[i] = iDSmooth(50+(-100)*(dMaxHigh - dClose) / (dMaxHigh - dMinLow),SmoothLength,SmoothPhase,i,20);
      }
      
      //
      //
      //
      //
      //
      
      for(i=limit; i>=0; i--) 
      {
         double deviations = iStdDevOnArray(wpr,0,BandsPeriod,0,MODE_SMA,i);
                bandsMi[i] = iMAOnArray(wpr,0,BandsPeriod,0,mamode,i);
                bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
                bandsDo[i] = bandsMi[i]-deviations*BandsDeviations;
                trend[i]   = trend[i+1];
                  if (wpr[i] > bandsMi[i] && wpr[i+1] < bandsMi[i+1]) trend[i] =  1;
                  if (wpr[i] < bandsMi[i] && wpr[i+1] > bandsMi[i+1]) trend[i] = -1;
       
         manageArrow(i);  
      }
      
      //
      //
      //
      //
      //
      
      manageAlerts(); 
      return(0);
   }

   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         wpr[i]    = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,0,y);
         bandsUp[i]= iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,1,y);
         bandsMi[i]= iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,2,y);
         bandsDo[i]= iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,3,y);
         cfb[i]    = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,4,y);
         trend[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"Calculatewpr",ShortLimit,LongLimit,CfbNormLength,CfbDepth,CfbPrice,CfbSmooth,SmoothLength,SmoothPhase,SmoothDouble,mamode,BandsPeriod,BandsDeviations,Delayed,5,y);
         manageArrow(i);

         //
         //
         //
         //
         //
        
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
        	      wpr[i+k]     = wpr[i]     + (wpr[i+n]-wpr[i])*k/n;
        	      bandsUp[i+k] = bandsUp[i] + (bandsUp[i+n]-bandsUp[i])*k/n;
        	      bandsMi[i+k] = bandsMi[i] + (bandsMi[i+n]-bandsMi[i])*k/n;
        	      bandsDo[i+k] = bandsDo[i] + (bandsDo[i+n]-bandsDo[i])*k/n;
            }  	            
   }

   //
   //
   //
   //
   //
         
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows && !Calculatingwpr)
   {
      double dist = iATR(NULL,0,20,i)/2.0;
      ObjectDelete(ArrowsIdentifier+Time[i]);         
            
      //
      //
      //
      //
      //
           
      atrend[i] = atrend[i+1];
      if (wpr[i] > bandsMi[i]) atrend[i] =  1;
      if (wpr[i] < bandsMi[i]) atrend[i] = -1;  
      if (atrend[i] != atrend[i+1])
      {
         string name = ArrowsIdentifier+Time[i];
         if (atrend[i] == 1)
         {
            ObjectCreate(name,OBJ_ARROW,0, Time[i],Low[i]-dist );
               ObjectSet(name,OBJPROP_ARROWCODE,241);
               ObjectSet(name,OBJPROP_COLOR,ArrowUpColor);
               ObjectSet(name,OBJPROP_WIDTH,ArrowUpWidth);
         }
         else
         {
            ObjectCreate(name,OBJ_ARROW,0, Time[i],High[i]+dist );
               ObjectSet(name,OBJPROP_ARROWCODE,242);
               ObjectSet(name,OBJPROP_COLOR,ArrowDownColor);
               ObjectSet(name,OBJPROP_WIDTH,ArrowDownWidth);
         }
      }
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!Calculatingwpr && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" cfb adaptive smooth wpr bands trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," cfb adaptive smooth wpr bands "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int    Depths[] = {2,3,4,6,8,12,16,24,32,48,64,96,128,192};
double workCfb[][28];

//
//
//
//
//

double iCfb(double price, int depth, int smooth, int r)
{
   if (ArrayRange(workCfb,0) != Bars) ArrayResize(workCfb,Bars);
   int i = Bars-r-1;
         
   //
   //
   //
   //
   //

   double suma     = 0;
   double sumb     = 0;
   double cfb      = 0;
   double evenCoef = 1;
   double oddCoef  = 1;
   double avg      = 0;
         
      if (i>=smooth)
         for (int k=depth-1; k>=0; k--)
         {
            workCfb[i][k]    = iCfbFunc(price,i,Depths[k],k);
            workCfb[i][k+14] = workCfb[i-1][k+14] + (workCfb[i][k]-workCfb[i-smooth][k])/smooth;

                  if ((k%2)==0)
                        { avg = oddCoef  * workCfb[i][k+14]; oddCoef  = oddCoef  * (1 - avg); }
                  else  { avg = evenCoef * workCfb[i][k+14]; evenCoef = evenCoef * (1 - avg); }
               
               suma += avg*avg*Depths[k];
               sumb += avg;
         }
      else for (k=depth-1; k>=0; k--) { workCfb[i][k] = 0; workCfb[i][k+14] = 0; }            

   //
   //
   //
   //
   //

   if (sumb != 0) cfb = suma/sumb;
   return(cfb);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double  workCfbFunc[][70];
#define _prices 0
#define _roc    1
#define _value1 2
#define _value2 3
#define _value3 4

//
//
//
//

double iCfbFunc(double price, int r, int depth, int k)
{
   k *= 5;
      if (ArrayRange(workCfbFunc,0) != Bars) ArrayResize(workCfbFunc,Bars);
      if (r<=(depth+1))
      {
         workCfbFunc[r][k+_prices] = 0;
         workCfbFunc[r][k+_roc]    = 0;
         workCfbFunc[r][k+_value1] = 0;
         workCfbFunc[r][k+_value2] = 0;
         workCfbFunc[r][k+_value3] = 0;
         return(0);
      }         
      workCfbFunc[r][k+_prices] = price; 

   //
   //
   //
   //
   //

      workCfbFunc[r][k+_roc]    = MathAbs(workCfbFunc[r][k+_prices] - workCfbFunc[r-1][k+_prices]);
      workCfbFunc[r][k+_value1] = workCfbFunc[r-1][k+_value1] - workCfbFunc[r-depth][k+_roc] + workCfbFunc[r][k+_roc];
      workCfbFunc[r][k+_value2] = workCfbFunc[r-1][k+_value2] - workCfbFunc[r-1][k+_value1] + workCfbFunc[r][k+_roc]*depth;
      workCfbFunc[r][k+_value3] = workCfbFunc[r-1][k+_value3] - workCfbFunc[r-1-depth][k+_prices] + workCfbFunc[r-1][k+_prices];
   
      double dividend = MathAbs(depth*workCfbFunc[r][k+_prices]-workCfbFunc[r][k+_value3]);

      //
      //
      //
      //
      //
         
   if (workCfbFunc[r][k+_value2] != 0)
         return( dividend / workCfbFunc[r][k+_value2]);
   else  return(0.00);            
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double wrk[][40];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

//
//
//
//
//

double iDSmooth(double price, double length, double phase, int i, int s=0)
{
   if (SmoothDouble)
         return (iSmooth(iSmooth(price,MathSqrt(length),phase,i,s),MathSqrt(length),phase,i,s+10));
   else  return (iSmooth(price,length,phase,i,s));
}

//
//
//
//
//

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (length <=1) return(price);
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrk[r][k+s]=price; for(; k<10; k++) wrk[r][k+s]=0; return(price); }

   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrk[r-1][bsmax+s];
      double del2   = price - wrk[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = MathMin(r,10);
	
         wrk[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
         wrk[r][vsum+s] =	wrk[r-1][vsum+s] + (wrk[r][volty+s]-wrk[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrk[r][avolty+s] = wrk[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrk[r][vsum+s]-wrk[r-1][avolty+s]);
            if (wrk[r][avolty+s] > 0)
               double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
         wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
         wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
         wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
         wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrk[r][4+s]);
}